#ifndef RESIP_QUERY_TYPES_HXX
#define RESIP_QUERY_TYPES_HXX

#include "rutil/HeapInstanceCounter.hxx"
#include "rutil/compat.hxx"
#include "rutil/Socket.hxx"
#include "rutil/BaseException.hxx"
#include "DnsResourceRecord.hxx"
#include "DnsHostRecord.hxx"
#include "DnsCnameRecord.hxx"
#include "DnsAAAARecord.hxx"
#include "DnsSrvRecord.hxx"
#include "DnsNaptrRecord.hxx"

//'Interface' of the RRType concept
// class QueryType
// {
//   public:
//       static unsigned short getRRType();
//       enum SupportsCName; //does following CNAME apply to this query type
// };


//!dcm! -- add UnusedChecking(_enum) below;

namespace resip
{

#define defineQueryType(_name, _type, _rrType, _supportsCName, _rfc)    \
class RR_##_name                                                        \
{                                                                       \
   public:                                                              \
      RESIP_HeapCount(RR_##_name);                                    \
      enum {SupportsCName = _supportsCName};                            \
      static unsigned short getRRType();                                \
      typedef _type Type;                                               \
      static Data getRRTypeName();                                          \
};                                                                      \
extern RR_##_name q_##_name

defineQueryType(A, DnsHostRecord, 1, true, "RFC 1035");
defineQueryType(CNAME, DnsCnameRecord, 5, false, "RFC 1035");

defineQueryType(AAAA, DnsAAAARecord, 28, true, "RFC 3596");

defineQueryType(SRV, DnsSrvRecord, 33, true, "RFC 2782");
defineQueryType(NAPTR, DnsNaptrRecord, 35, true, "RFC 2915");

}

#undef defineQueryType

#endif

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000-2005 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
 