#if !defined(RESIP_HANDLE_HXX)
#define RESIP_HANDLE_HXX

#include <iosfwd>
#include "rutil/ResipAssert.h"
#include "resip/dum/Handled.hxx"
#include "resip/dum/HandleManager.hxx"
#include "resip/dum/HandleException.hxx"

namespace resip
{

template <class T>
class Handle
{
   public:
      Handle(HandleManager& ham, Handled::Id id) : mHam(&ham), mId(id)
      {
      }

      Handle() : mHam(0), mId(0)
      {
      }

      bool isValid() const
      {
         if (!mHam)
         {
            return false;
         }
         else
         {
            return mHam->isValidHandle(mId);
         }
      }
      
      // throws if not found
      T* get()
      {
         if (!mHam)
         {
            //assert(0);
            throw HandleException("Reference to unitialized handle.", __FILE__, __LINE__);
         }
         return static_cast<T*>(mHam->getHandled(mId));
      }

      const T* get() const
      {
         if (!mHam)
         {
            //assert(0);
            throw HandleException("Reference to unitialized handle.", __FILE__, __LINE__);
         }
         return static_cast<T*>(mHam->getHandled(mId));
      }
      
      T* operator->()
      {
         return get();
      }

      const T* operator->() const
      {
         return get();
      }

      T& operator*()
      {
         return *get();
      }

      const T& operator*() const
      {
         return *get();
      }
      

      Handled::Id getId() const
      {
         return mId;
      }

      static Handle<T> NotValid()
      {
         static Handle<T> notValid;
         return notValid;
      }
      
      bool operator==(const Handle<T>& other)
      {
         return (mHam == other.mHam) && (mId == other.mId);
      }
      
      // !nash! to be able to use Handle in Set or Map container
      bool operator<(const Handle<T>& other) const
      {
         resip_assert(mHam);
         resip_assert(other.mHam);
         return mId < other.mId;
      }

   private:
      HandleManager* mHam;

   protected:
      Handled::Id mId;

      friend class Handled;
};

}

#endif

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
