#ifndef RESIP_DNS_NAPTR_RECORD
#define RESIP_DNS_NAPTR_RECORD

#include "rutil/Data.hxx"
#include "rutil/Socket.hxx"
#include "rutil/BaseException.hxx"
#include "rutil/dns/DnsResourceRecord.hxx"

namespace resip
{

class Data;
class DnsResourceRecord;
class RROverlay;
class BaseException;

class DnsNaptrRecord : public DnsResourceRecord
{
   public:
      class NaptrException : public BaseException
      {
         public:
            NaptrException(const Data& msg, const Data& file, const int line)
               : BaseException(msg, file, line) 
            {
            }
            
            const char* name() const { return "NaptrException"; }
      };

      class RegExp
      {
         public:
            // Takes a regexp expression as defined in rfc 2915 (section 3
            // Substitution Expression Grammar) The delimiter is whatever
            // appears in the first character. This can be empty. 
            RegExp(const Data& data);
            RegExp();
            ~RegExp();
            
            bool empty() const;
            
            // Convenience method provided for access to the antecedent of the
            // regexp (the matching regular expression) 
            const Data& regexp() const;

            // Convenience method provided for access to the consequent of the
            // regexp (the replacement)  - this must be a URI
            const Data& replacement() const;
            
            // Applies the regular expression substitution based on the input
            // string. Will return Data::Empty if the input does not match the
            // substitution. 
            Data apply(const Data& input) const;
            
         private:
            Data mRegexp;
            Data mReplacement;
            Data flags;
            //regex_t mRe;
      };
      
         
      DnsNaptrRecord() : mOrder(-1), mPreference(-1) {}
      DnsNaptrRecord(const RROverlay&);
      ~DnsNaptrRecord() {}

      // accessors.
      int order() const { return mOrder; }
      int& order() { return mOrder; }
      int preference() const { return mPreference; }
      const Data& flags() const { return mFlags; }
      const Data& service() const { return mService; }
      const RegExp& regexp() const { return mRegexp; }
      const Data& replacement() const { return mReplacement; }
      const Data& name() const { return mName; }
      bool isSameValue(const Data& value) const;
      EncodeStream& dump(EncodeStream& strm) const;
      
   private:
      int mOrder;
      int mPreference;
      Data mFlags;
      Data mService;
      RegExp mRegexp;
      Data mReplacement;
      Data mName;

};

}


#endif

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000-2005 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
